# Generated by roxygen2: do not edit by hand

S3method(GOF,bsim)
S3method(as.data.frame,bsimPred)
S3method(as_bsim,bsimSetup)
S3method(coef,bsim)
S3method(fitted,bsim)
S3method(plot,bsim)
S3method(plot,bsimPred)
S3method(plot,bsimSetup)
S3method(predict,bsim)
S3method(print,bsim)
S3method(print,bsimPred)
S3method(print,bsimSetup)
S3method(print,summary.bsim)
S3method(print,summary.bsimPred)
S3method(print,summary.bsimSetup)
S3method(residuals,bsim)
S3method(summary,bsim)
S3method(summary,bsimPred)
S3method(summary,bsimSetup)
export(BayesSIM)
export(BayesSIM_setup)
export(GOF)
export(as_bsim)
export(bsFisher)
export(bsFisher_setup)
export(bsPolar)
export(bsPolar_setup)
export(bsSphere)
export(bsSphere_setup)
export(bsSpike)
export(bsSpike_setup)
export(compileModelAndMCMC)
export(getInit)
export(getModel)
export(getModelDef)
export(getSampler)
export(getVarMonitor)
export(gpFisher)
export(gpFisher_setup)
export(gpPolar)
export(gpPolarHigh)
export(gpPolarHigh_setup)
export(gpPolar_setup)
export(gpSphere)
export(gpSphere_setup)
export(gpSpike)
export(gpSpike_setup)
export(init_param)
export(nimTraceplot)
export(prior_param)
import(ggplot2)
import(methods)
import(nimble)
import(patchwork)
importFrom(MASS,mvrnorm)
importFrom(dplyr,select)
importFrom(ggplot2,aes)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(magrittr,"%>%")
importFrom(mvtnorm,rmvnorm)
importFrom(stats,dbeta)
importFrom(stats,dnorm)
importFrom(stats,knots)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,pnorm)
importFrom(stats,ppr)
importFrom(stats,qnorm)
importFrom(stats,qpois)
importFrom(stats,quantile)
importFrom(stats,rbeta)
importFrom(stats,rbinom)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(tidyr,all_of)
importFrom(tidyr,pivot_longer)
importFrom(utils,modifyList)
importFrom(utils,str)
